#! /usr/bin/env python
"""PyOpenGL setup script (setuptools-based)
"""
import sys, os
extra_commands = {}
try:
    from setuptools import setup 
except ImportError:
    from distutils.core import setup

HERE = os.path.normpath(os.path.abspath(os.path.dirname( __file__ )))
with open(os.path.join(HERE, 'readme.rst'), 'r') as f:
    long_description = f.read()

sys.path.insert(0, '.' )
metadata = dict(
    version = [
        (line.split('=')[1]).strip().strip('"').strip("'")
        for line in open(os.path.join('OpenGL','version.py'))
        if line.startswith( '__version__' )
    ][0],
    author = 'Mike C. Fletcher',
    author_email = 'mcfletch@vrplumber.com',
    url = 'http://pyopengl.sourceforge.net',
    license = 'BSD',
    download_url = "http://sourceforge.net/projects/pyopengl/files/PyOpenGL/",
    keywords = 'Graphics,3D,OpenGL,GLU,GLUT,GLE,GLX,EXT,ARB,Mesa,ctypes',
    classifiers = [
        """License :: OSI Approved :: BSD License""",
        """Programming Language :: Python""",
        """Programming Language :: Python :: 3""",
        """Topic :: Multimedia :: Graphics :: 3D Rendering""",
        """Topic :: Software Development :: Libraries :: Python Modules""",
        """Intended Audience :: Developers""",
    ],
    long_description = long_description,
    long_description_content_type = 'text/x-rst',
)

def is_package( path ):
    return os.path.isfile( os.path.join( path, '__init__.py' ))
def find_packages( root ):
    """Find all packages under this directory"""
    for path, directories, files in os.walk( root ):
        if is_package( path ):
            yield path.replace( '/','.' )

requirements = []
if sys.hexversion < 0x2050000:
    requirements.append( 'ctypes' )

from distutils.command.install_data import install_data
class smart_install_data(install_data):
    def run(self):
        #need to change self.install_dir to the library dir
        install_cmd = self.get_finalized_command('install')
        self.install_dir = getattr(install_cmd, 'install_lib')
        # should create the directory if it doesn't exist!!!
        return install_data.run(self)
extra_commands['install_data'] = smart_install_data

if sys.platform == 'win32':
    # binary versions of GLUT and GLE for Win32 (sigh)
    DLL_DIRECTORY = os.path.join('OpenGL','DLLS')
    datafiles = [
        (
            DLL_DIRECTORY, [
                os.path.join( DLL_DIRECTORY,file)
                for file in os.listdir( DLL_DIRECTORY )
                if os.path.isfile( os.path.join( DLL_DIRECTORY,file) )
            ]
        ),
    ]
else:
    datafiles = []


if __name__ == "__main__":
    setup(
        name = "PyOpenGL",
        packages = list( find_packages('OpenGL') ),
        description = 'Standard OpenGL bindings for Python',
        options = {
            'sdist': {
                'formats': ['gztar','zip'],
                'force_manifest': True,
            },
        },
        data_files = datafiles,
        cmdclass = extra_commands,
        **metadata
    )



import basetestcase
import os
from OpenGL.GL import *
try:
    import numpy as np
except ImportError as err:
    np = None
from OpenGL.arrays import arraydatatype
HERE = os.path.abspath(os.path.dirname(__file__))
from OpenGL.GL.ARB import texture_rg

class TestTextures(basetestcase.BaseTest):
    def test_enable_histogram( self ):
        if glInitImagingARB():
            glHistogram(GL_HISTOGRAM, 256, GL_LUMINANCE, GL_FALSE)
            glEnable( GL_HISTOGRAM )
            glDisable( GL_HISTOGRAM )
        else:
            print('No ARB imaging extension')
    if np:
        def test_glreadpixels_warray( self ):
            """SF#1311265 allow passing in the array object"""
            width,height = self.width, self.height
            data = np.zeros( (width,height,3), 'B' )
            image1 = glReadPixelsub(0,0,width,height,GL_RGB,array=data)
            assert image1 is not None
        someData = [ (0,255,0)]
        def test_glAreTexturesResident( self ):
            """Test that PyOpenGL api for glAreTexturesResident is working
            
            Note: not currently working on AMD64 Linux for some reason
            """
            textures = glGenTextures(2)
            residents = []
            data = np.array( self.someData,'i' )
            for texture in textures:
                glBindTexture( GL_TEXTURE_2D,int(texture) )
                glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB8, 1, 1, 0, GL_RGB, GL_INT, data)
                residents.append(
                    glGetTexParameteriv(GL_TEXTURE_2D, GL_TEXTURE_RESIDENT )
                )
            glGetError()
            result = glAreTexturesResident( textures)
            assert len(result) == 2
            for (tex,expected,found) in zip( textures, residents, result ):
                if expected != found:
                    print(('Warning: texture %s residence expected %s got %s'%( tex, expected, found )))
    def test_glreadpixelsf( self ):
        """Issue #1979002 crash due to mis-calculation of resulting array size"""
        width,height = self.width, self.height
        readback_image1 = glReadPixelsub(0,0,width,height,GL_RGB)
        assert readback_image1 is not None
        readback_image2 = glReadPixelsf(0,0,width,height,GL_RGB)
        assert readback_image2 is not None
    def test_glreadpixels_is_string( self ):
        """Issue #1959860 incompatable change to returning arrays reversed"""
        width,height = self.width, self.height
        readback_image1 = glReadPixels(0,0,width,height,GL_RGB, GL_UNSIGNED_BYTE)
        assert isinstance( readback_image1, bytes ), type( readback_image1 )
        readback_image1 = glReadPixels(0,0,width,height,GL_RGB, GL_BYTE)
        assert not isinstance( readback_image1, bytes ), type(readback_image2)
    def test_passBackResults( self ):
        """Test ALLOW_NUMPY_SCALARS to allow numpy scalars to be passed in"""
        textures = glGenTextures(2)
        glBindTexture( GL_TEXTURE_2D, textures[0] )
    def test_nullTexture( self ):
        """Test that we can create null textures"""
        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB8, 512, 512, 0, GL_RGB, GL_INT, None)

    def test_get_boolean_bitmap( self ):
        # should not raise error
        glGetBoolean(GL_TEXTURE_2D)
    if np:
        def test_draw_bitmap_pixels( self ):
            """SF#2152623 Drawing pixels as bitmaps (bits)"""
            # this core-dumps on Mesa Intel on Ubuntu 15.04 :(
            # nosetest skip would be more appropriate
            return False
            pixels = np.array([0,0,0,0,0,0,0,0],'B')
            glDrawPixels( 8,8, GL_COLOR_INDEX, GL_BITMAP, pixels )

    def test_get_max_tex_units( self ):
        """SF#2895081 glGetIntegerv( GL_MAX_TEXTURE_IMAGE_UNITS )"""
        units = glGetIntegerv( GL_MAX_TEXTURE_IMAGE_UNITS )
        assert units
    def test_glGenTextures( self ):
        texture = glGenTextures(1)
        assert texture
        
    def test_createTargetArray(self):
        import OpenGL.GL as gl
        import OpenGL.images as images
        size = (640,480)
        array1 = images.createTargetArray( gl.GL_BGRA, size, gl.GL_UNSIGNED_INT_8_8_8_8_REV)
        array2 = images.createTargetArray( gl.GL_RGBA, size, gl.GL_UNSIGNED_BYTE)
        array3 = images.createTargetArray( gl.GL_RGBA, size, gl.GL_UNSIGNED_INT_8_8_8_8_REV)
        if hasattr( array1, 'nbytes'):
            assert array1.nbytes == array3.nbytes
            assert array1.nbytes == array2.nbytes
        else:
            assert ctypes.sizeof( array1 ) == ctypes.sizeof(array3)
            assert ctypes.sizeof( array1 ) == ctypes.sizeof(array2)
        
        try:
            images.createTargetArray( gl.GL_RGBA, size, gl.GL_UNSIGNED_BYTE_3_3_2 )
        except ValueError as err:
            pass 
        else:
            raise RuntimeError( """Should have failed with insufficient components in the type to hold the format""" )
    

    def test_rg_format(self):
        # Note: this is actually only known after context creation...
        if not texture_rg.glInitTextureRgARB():
            return 
        texture = glGenTextures(1)
        data = arraydatatype.GLfloatArray.asArray([.3,.5])
        glBindTexture( GL_TEXTURE_2D,int(texture) )
        glTexImage2D(GL_TEXTURE_2D, 0, texture_rg.GL_RG, 1, 1, 0, GL_RG, GL_FLOAT, data)
    
        



"""Test for github issue #47"""
from __future__ import print_function
import OpenGL
OpenGL.SIZE_1_ARRAY_UNPACK = False # just for convenience
OpenGL.ERROR_ON_COPY = False # we are checking a leak in the copying
import pygamegltest
import numpy as np
from OpenGL.GL import *
from OpenGL.GL import shaders
from sys import getrefcount

@pygamegltest.pygametest(name="Texture image 2d leak check")
def main():
    data = np.zeros([256,256,3],dtype='b')
    glEnable(GL_TEXTURE_2D)
    textures = glGenTextures(1)
    glBindTexture(GL_TEXTURE_2D, textures[0])
    reversed_data = data[::-1]
    assert not reversed_data.flags['C_CONTIGUOUS']
    rc1 = getrefcount(reversed_data)
    for i in range(100):
        glTexImage2D(
            GL_TEXTURE_2D, 
            0, 
            GL_RGB, 
            256, 256, 0, GL_RGB,GL_UNSIGNED_BYTE, reversed_data)
    rc2 = getrefcount(reversed_data)
    assert rc1 == rc2, (rc1, rc2)


if __name__ == "__main__":
    main()



import pygame, sys
from pygame.locals import *
import OpenGL
OpenGL.USE_ACCELERATE = False
from OpenGL.GL import *
from OpenGL.GL.EXT.framebuffer_object import *
from OpenGL import error

def draw ():
    glClearColor(0.0,0.0,0.0,0.0)
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
    #draw stuff here
    pygame.display.flip()

def main():
    pygame.init()
    pygame.display.set_mode((512,512),OPENGL | DOUBLEBUF)

    #setup a texture
    tex = glGenTextures(1);
    glBindTexture(GL_TEXTURE_2D, tex);
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, 512, 512, 0, GL_RGBA,
    GL_UNSIGNED_BYTE, None);
    glBindTexture(GL_TEXTURE_2D, 0);

    #setup teh fbo
    fbo = glGenFramebuffersEXT(1)
    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, fbo)
    glBindTexture(GL_TEXTURE_2D, tex)

    #this call produces an error!

    glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT,
    GL_COLOR_ATTACHMENT0_EXT,GL_TEXTURE_2D, tex, 0)

    while 1:
        event=pygame.event.poll ()
        if event.type is QUIT:
            sys.exit(0)
        draw()

if __name__ == "__main__":
    main()



#! /usr/bin/env python
"""Build the (Open)GLE distribution

Start a VC shell:
    
    "c:\Program Files (x86)\Microsoft Visual Studio 9.0\VC\bin\Visual Studio 2008 Command Prompt.lnk"
    "c:\Program Files (x86)\Microsoft Visual Studio 9.0\VC\bin\Visual Studio 2008 x64 Win64 Command Prompt.lnk"

Then run this script with the appropriate (32-bit or 64-bit python):

    c:\python27-32\python.exe buildgledll.py
    c:\python27-64\python.exe buildgledll.py

Note: the name "opengle" is required because someone issues DMCA takedown orders against anything named "gle.dll"
despite the name GLE referring to the GLE project for a very long time (they took PyOpenGL offline for a while
due to such a takedown notice).
"""
from __future__ import print_function
import sys, os, subprocess, requests, logging, platform, shutil, glob
log = logging.getLogger( 'buildgle' )

DOWNLOAD_URL = 'http://downloads.sourceforge.net/project/gle/gle/gle-3.1.0/gle-3.1.0.tar.gz?r=http%3A%2F%2Fsourceforge.net%2Fprojects%2Fgle%2F&ts=1315332658&use_mirror=voxel'
GLE_VERSION = '3.1.0'
GLE_SOURCE_DIR = 'gle-%(GLE_VERSION)s'%globals()
TAR_FILE = '%(GLE_SOURCE_DIR)s.tar.gz'%globals()
EXPORTS = """
gleExtrusion gleGetNumSides gleSetJoinStyle glePolyCylinder gleSpiral gleSetNumSides uview_direction gleScrew
gleHelicoid gleToroid gleExtrusion gleTextureMode gleSuperExtrusion gleLathe gleGetJoinStyle glePolyCone gleTwistExtrusion 
urot_omega rot_about_axis urot_prince rot_prince urot_about_axis rot_omega rot_axis uviewpoint urot_axis"""

if sys.hexversion < 0x2070000:
    VC = 'vc7'
# TODO: add Python 3.x compiler compatibility...
else:
    VC = 'vc9'

def download_and_unpack():
    if not os.path.exists( TAR_FILE ):
        response = requests.get( DOWNLOAD_URL )
        if response.ok:
            open( TAR_FILE, 'wb').write( response.content )
    if not os.path.exists( GLE_SOURCE_DIR ):
        subprocess.check_call( 'tar -zxvf %(TAR_FILE)s'%globals() )

def size():
    return platform.architecture()[0].strip( 'bits' )

def build():
    current = os.getcwd()
    suffix = size()
    shutil.copyfile( os.path.join( GLE_SOURCE_DIR, 'ms-visual-c', 'config.h' ), os.path.join(GLE_SOURCE_DIR,'config.h') )
    shutil.copyfile( os.path.join( GLE_SOURCE_DIR, 'ms-visual-c', 'config.h' ), os.path.join(GLE_SOURCE_DIR,'src','config.h') )
    os.chdir( os.path.join( GLE_SOURCE_DIR, 'src' ))
    try:
        for file in glob.glob( '*.obj' ):
            os.remove( file )
        for file in glob.glob( '*%(suffix)s.dll'%locals() ):
            os.remove( file )

        vc = VC
        outfile = 'opengle%(suffix)s.%(vc)s.dll'%locals()
        target = os.path.join( current, '..', 'OpenGL','DLLS', outfile )
        exports = " ".join([ '/EXPORT:%s'%(x) for x in EXPORTS.split() if x])

        subprocess.check_call( 'cl -c /D"WIN32" /D "_WINDLL" /Gd /MD *.c' )
        subprocess.check_call( 'link  /LIBPATH:"C:\Program Files\Microsoft Platform SDK\Lib" %(exports)s /DLL /OUT:%(outfile)s opengl32.lib glu32.lib *.obj'%locals() )

        shutil.copyfile( outfile, target )
        print('Created file %(target)s'%locals())
        
    finally:
        os.chdir( current )

def main():
    download_and_unpack()
    build()

if __name__ == "__main__":
    logging.basicConfig( level=logging.INFO )
    main()
